@@ -61,25 +61,6 @@ _MODEL_SAVE_PATH = "model"
 _PIP_ENV_SUBPATH = "requirements.txt"
 
 
-def _raise_deprecation_warning():
-    # Avoid `ModuleNotFoundError` thrown when this function is called from `mlflow.tensorflow` to
-    # save a model created using `tensorflow.keras` but `keras` is not installed.
-    try:
-        import keras
-    except ImportError:
-        return
-
-    if Version(keras.__version__) < Version("2.3.0"):
-        warnings.warn(
-            (
-                "Support for keras < 2.3.0 has been deprecated and will be removed in a future "
-                "MLflow release"
-            ),
-            FutureWarning,
-            stacklevel=2,
-        )
-
-
 def get_default_pip_requirements(include_cloudpickle=False, keras_module=None):
     """
     :return: A list of default pip requirements for MLflow Models produced by this flavor.
@@ -190,7 +171,6 @@ def save_model(
         # Save the model as an MLflow Model
         mlflow.keras.save_model(keras_model, keras_model_path)
     """
-    _raise_deprecation_warning()
     _validate_env_arguments(conda_env, pip_requirements, extra_pip_requirements)
 
     if keras_module is None:
@@ -199,15 +179,10 @@ def save_model(
             try:
                 import keras
 
-                if Version(keras.__version__) < Version("2.2.0"):
-                    import keras.engine
+                # NB: Network is the first parent with save method
+                import keras.engine.network
 
-                    return isinstance(model, keras.engine.Model)
-                else:
-                    # NB: Network is the first parent with save method
-                    import keras.engine.network
-
-                    return isinstance(model, keras.engine.network.Network)
+                return isinstance(model, keras.engine.network.Network)
             except ImportError:
                 return False
 
@@ -505,8 +480,6 @@ def _load_pyfunc(path):
 
     :param path: Local filesystem path to the MLflow Model with the ``keras`` flavor.
     """
-    import tensorflow as tf
-
     if os.path.isfile(os.path.join(path, _KERAS_MODULE_SPEC_PATH)):
         with open(os.path.join(path, _KERAS_MODULE_SPEC_PATH), "r") as f:
             keras_module = importlib.import_module(f.read())
@@ -526,28 +499,11 @@ def _load_pyfunc(path):
     should_compile = save_format == "tf"
     K = importlib.import_module(keras_module.__name__ + ".backend")
     if keras_module.__name__ == "tensorflow.keras" or K.backend() == "tensorflow":
-        if Version(tf.__version__) < Version("2.0.0"):
-            graph = tf.Graph()
-            sess = tf.Session(graph=graph)
-            # By default tf backed models depend on the global graph and session.
-            # We create an use new Graph and Session and store them with the model
-            # This way the model is independent on the global state.
-            with graph.as_default():
-                with sess.as_default():  # pylint:disable=not-context-manager
-                    K.set_learning_phase(0)
-                    m = _load_model(
-                        path,
-                        keras_module=keras_module,
-                        save_format=save_format,
-                        compile=should_compile,
-                    )
-                    return _KerasModelWrapper(m, graph, sess)
-        else:
-            K.set_learning_phase(0)
-            m = _load_model(
-                path, keras_module=keras_module, save_format=save_format, compile=should_compile
-            )
-            return _KerasModelWrapper(m, None, None)
+        K.set_learning_phase(0)
+        m = _load_model(
+            path, keras_module=keras_module, save_format=save_format, compile=should_compile
+        )
+        return _KerasModelWrapper(m, None, None)
 
     else:
         raise MlflowException("Unsupported backend '%s'" % K._BACKEND)
@@ -581,7 +537,6 @@ def load_model(model_uri, **kwargs):
         keras_model = mlflow.keras.load_model("runs:/96771d893a5e46159d9f3b49bf9013e2" + "/models")
         predictions = keras_model.predict(x_test)
     """
-    _raise_deprecation_warning()
     local_model_path = _download_artifact_from_uri(artifact_uri=model_uri)
     flavor_conf = _get_flavor_configuration(model_path=local_model_path, flavor_name=FLAVOR_NAME)
     keras_module = importlib.import_module(flavor_conf.get("keras_module", "keras"))
@@ -671,7 +626,15 @@ def autolog(
     """
     import keras
 
-    _raise_deprecation_warning()
+    if Version(keras.__version__) >= Version("2.6.0"):
+        warnings.warn(
+            (
+                "Autologging support for keras >= 2.6.0 has been deprecated and will be removed in "
+                "a future MLflow release. Use `mlflow.tensorflow.autolog()` instead."
+            ),
+            FutureWarning,
+            stacklevel=2,
+        )
 
     def getKerasCallback(metrics_logger):
         class __MLflowKerasCallback(keras.callbacks.Callback, metaclass=ExceptionSafeClass):
@@ -737,9 +700,7 @@ def autolog(
         return __MLflowKerasCallback()
 
     def _early_stop_check(callbacks):
-        if Version(keras.__version__) < Version("2.3.0") or Version(keras.__version__) >= Version(
-            "2.4.0"
-        ):
+        if Version(keras.__version__) >= Version("2.4.0"):
             es_callback = keras.callbacks.EarlyStopping
         else:
             es_callback = keras.callbacks.callbacks.EarlyStopping
