@@ -1,13 +1,19 @@
-from collections import defaultdict, namedtuple
+from collections import defaultdict, namedtuple, OrderedDict
 import logging
 import numpy as np
 import time
 from pkg_resources import resource_filename
+import weakref
 
 import mlflow
 from mlflow.entities import Metric, Param
 from mlflow.tracking.client import MlflowClient
-from mlflow.utils import _chunk_dict, _truncate_dict, _get_fully_qualified_class_name
+from mlflow.utils import (
+    _chunk_dict,
+    _truncate_dict,
+    _get_fully_qualified_class_name,
+    _inspect_original_var_name,
+)
 from mlflow.utils.annotations import experimental
 from mlflow.utils.autologging_utils import (
     _get_new_training_session_class,
@@ -15,6 +21,7 @@ from mlflow.utils.autologging_utils import (
     safe_patch,
     try_mlflow_log,
 )
+from mlflow.utils.autologging_utils import get_method_call_arg_value
 from mlflow.utils.file_utils import TempDir
 from mlflow.utils.mlflow_tags import MLFLOW_AUTOLOGGING, MLFLOW_PARENT_RUN_ID
 from mlflow.utils.validation import (
@@ -139,8 +146,8 @@ def _should_log_hierarchy(estimator):
     return isinstance(estimator, (Pipeline, OneVsRest)) or _is_parameter_search_estimator(estimator)
 
 
-AutologgingEstimatorMetadata = namedtuple(
-    "AutologgingEstimatorMetadata",
+_AutologgingEstimatorMetadata = namedtuple(
+    "_AutologgingEstimatorMetadata",
     ["hierarchy", "uid_to_indexed_name_map", "param_search_estimators"],
 )
 
@@ -151,7 +158,15 @@ def _traverse_stage(stage):
 
     yield stage
     if isinstance(stage, Pipeline):
-        for stage in stage.getStages():
+        original_sub_stages = stage.getStages()
+
+        try:
+            iter(original_sub_stages)
+        except TypeError:
+            raise TypeError(
+                f"Pipeline stages should be iterable, but found object {original_sub_stages}"
+            )
+        for stage in original_sub_stages:
             yield from _traverse_stage(stage)
     elif isinstance(stage, OneVsRest):
         yield from _traverse_stage(stage.getClassifier())
@@ -208,8 +223,8 @@ def _gen_stage_hierarchy_recursively(
 
 def _gen_estimator_metadata(estimator):
     """
-    Returns an AutologgingEstimatorMetadata object.
-    The AutologgingEstimatorMetadata object includes:
+    Returns an `_AutologgingEstimatorMetadata` object.
+    The `_AutologgingEstimatorMetadata` object includes:
      - hierarchy: the hierarchy of the estimator, it will expand
          pipeline/meta estimator/param tuning estimator
      - uid_to_indexed_name_map: a map of `uid` -> `name`, each nested instance uid will be
@@ -225,7 +240,7 @@ def _gen_estimator_metadata(estimator):
     ]
     hierarchy = _gen_stage_hierarchy_recursively(estimator, uid_to_indexed_name_map)
 
-    metadata = AutologgingEstimatorMetadata(
+    metadata = _AutologgingEstimatorMetadata(
         hierarchy=hierarchy,
         uid_to_indexed_name_map=uid_to_indexed_name_map,
         param_search_estimators=param_search_estimators,
@@ -235,15 +250,19 @@ def _gen_estimator_metadata(estimator):
     return metadata
 
 
-def _get_instance_param_map_recursively(instance, level, uid_to_indexed_name_map):
-    from pyspark.ml.param import Params
-    from pyspark.ml.pipeline import Pipeline
-
-    param_map = {
+def _get_param_map(instance):
+    return {
         param.name: instance.getOrDefault(param)
         for param in instance.params
         if instance.isDefined(param)
     }
+
+
+def _get_instance_param_map_recursively(instance, level, uid_to_indexed_name_map):
+    from pyspark.ml.param import Params
+    from pyspark.ml.pipeline import Pipeline
+
+    param_map = _get_param_map(instance)
     expanded_param_map = {}
 
     is_pipeline = isinstance(instance, Pipeline)
@@ -307,7 +326,7 @@ def _create_child_runs_for_parameter_search(parent_estimator, parent_model, pare
     estimator_param_maps = parent_estimator.getEstimatorParamMaps()
     tuned_estimator = parent_estimator.getEstimator()
 
-    metric_key, metrics = _get_param_search_metrics(parent_estimator, parent_model)
+    metrics_dict, _ = _get_param_search_metrics_and_best_index(parent_estimator, parent_model)
     for i in range(len(estimator_param_maps)):
         child_estimator = tuned_estimator.copy(estimator_param_maps[i])
         tags_to_log = dict(child_tags) if child_tags else {}
@@ -324,7 +343,7 @@ def _create_child_runs_for_parameter_search(parent_estimator, parent_model, pare
             child_estimator, parent_estimator._autologging_metadata.uid_to_indexed_name_map
         )
         param_batches_to_log = _chunk_dict(params_to_log, chunk_size=MAX_PARAMS_TAGS_PER_BATCH)
-        metrics_to_log = {metric_key: metrics[i]}
+        metrics_to_log = {k: v[i] for k, v in metrics_dict.items()}
         for params_batch, metrics_batch in zip_longest(
             param_batches_to_log, [metrics_to_log], fillvalue={}
         ):
@@ -349,13 +368,13 @@ def _create_child_runs_for_parameter_search(parent_estimator, parent_model, pare
         client.set_terminated(run_id=child_run.info.run_id, end_time=child_run_end_time)
 
 
-def _log_parameter_search_results_as_artifact(param_maps, metrics, metric_name, run_id):
+def _log_parameter_search_results_as_artifact(param_maps, metrics_dict, run_id):
     import pandas as pd
     import json
 
     result_dict = defaultdict(list)
     result_dict["params"] = []
-    result_dict[metric_name] = metrics
+    result_dict.update(metrics_dict)
     for param_map in param_maps:
         result_dict["params"].append(json.dumps(param_map))
         for param_name, param_value in param_map.items():
@@ -379,29 +398,52 @@ def _get_warning_msg_for_fit_call_with_a_list_of_params(estimator):
 
 def _get_tuning_param_maps(param_search_estimator, uid_to_indexed_name_map):
     tuning_param_maps = []
+
+    def gen_log_key(param):
+        if param.parent not in uid_to_indexed_name_map:
+            raise ValueError(
+                "Tuning params should not include params not owned by the tuned estimator, but "
+                f"found a param {param}"
+            )
+        return f"{uid_to_indexed_name_map[param.parent]}.{param.name}"
+
     for eps in param_search_estimator.getEstimatorParamMaps():
-        tuning_param_maps.append(
-            {f"{uid_to_indexed_name_map[k.parent]}.{k.name}": v for k, v in eps.items()}
-        )
+        tuning_param_maps.append({gen_log_key(k): v for k, v in eps.items()})
     return tuning_param_maps
 
 
-def _get_param_search_metrics(param_search_estimator, param_search_model):
+def _get_param_search_metrics_and_best_index(param_search_estimator, param_search_model):
     """
-    Return a tuple of (metric_key, metrics: Array)
+    Return a tuple of `(metrics_dict, best_index)`
+    `metrics_dict` is a dict of metric_name --> metric_values for each param map
+    - For CrossValidatorModel, the result dict contains metrics of avg_metris and std_metrics
+      for each param map.
+    - For TrainValidationSplitModel, the result dict contains metrics for each param map.
+
+    `best_index` is the best index of trials.
     """
     from pyspark.ml.tuning import CrossValidatorModel, TrainValidationSplitModel
 
+    metrics_dict = {}
+
     metric_key = param_search_estimator.getEvaluator().getMetricName()
     if isinstance(param_search_model, CrossValidatorModel):
-        metric_key = "avg_" + metric_key
-        metrics = param_search_model.avgMetrics
+        avg_metrics = param_search_model.avgMetrics
+        metrics_dict["avg_" + metric_key] = avg_metrics
+        if hasattr(param_search_model, "stdMetrics"):
+            metrics_dict["std_" + metric_key] = param_search_model.stdMetrics
     elif isinstance(param_search_model, TrainValidationSplitModel):
-        metrics = param_search_model.validationMetrics
+        avg_metrics = param_search_model.validationMetrics
+        metrics_dict[metric_key] = avg_metrics
     else:
         raise RuntimeError(f"Unknown parameter search model type {type(param_search_model)}.")
 
-    return metric_key, metrics
+    if param_search_estimator.getEvaluator().isLargerBetter():
+        best_index = np.argmax(avg_metrics)
+    else:
+        best_index = np.argmin(avg_metrics)
+
+    return metrics_dict, best_index
 
 
 def _log_estimator_params(param_map):
@@ -411,6 +453,226 @@ def _log_estimator_params(param_map):
         try_mlflow_log(mlflow.log_params, truncated)
 
 
+class _AutologgingMetricsManager:
+    """
+    This class is designed for holding information which is used by autologging metrics
+    It will hold information of:
+    (1) a map of "prediction result object id" to a tuple of dataset name(the dataset is
+       the one which generate the prediction result) and run_id.
+       Note: We need this map instead of setting the run_id into the "prediction result object"
+       because the object maybe a numpy array which does not support additional attribute
+       assignment.
+    (2) _log_post_training_metrics_enabled flag, in the following method scope:
+       `Estimator.fit`, `Model.transform`, `Evaluator.evaluate`,
+       in order to avoid nested/duplicated autologging metric, when run into these scopes,
+       we need temporarily disable the metric autologging.
+    (3) _eval_dataset_info_map, it is a double level map:
+       `_eval_dataset_info_map[run_id][eval_dataset_var_name]` will get a list, each
+       element in the list is an id of "eval_dataset" instance.
+       This data structure is used for:
+        * generating unique dataset name key when autologging metric. For each eval dataset object,
+          if they have the same eval_dataset_var_name, but object ids are different,
+          then they will be assigned different name (via appending index to the
+          eval_dataset_var_name) when autologging.
+    (4) _evaluator_call_info, it is a double level map:
+       `_metric_api_call_info[run_id][metric_name]` wil get a list of tuples, each tuple is:
+         (logged_metric_key, evaluator_information)
+        Evaluator information includes evaluator class name and params, these information
+        will also be logged into "metric_info.json" artifacts.
+
+    Note: this class is not thread-safe.
+    Design rule for this class:
+     Because this class instance is a global instance, in order to prevent memory leak, it should
+     only holds IDs and other small objects references. This class internal data structure should
+     avoid reference to user dataset variables or model variables.
+    """
+
+    def __init__(self):
+        self._pred_result_id_to_dataset_name_and_run_id = {}
+        self._eval_dataset_info_map = defaultdict(lambda: defaultdict(list))
+        self._evaluator_call_info = defaultdict(lambda: defaultdict(list))
+        self._log_post_training_metrics_enabled = True
+        self._metric_info_artifact_need_update = defaultdict(lambda: False)
+
+    def should_log_post_training_metrics(self):
+        """
+        Check whether we should run patching code for autologging post training metrics.
+        This checking should surround the whole patched code due to the safe guard checking,
+        See following note.
+
+        Note: It includes checking `_SparkTrainingSession.is_active()`, This is a safe guarding
+        for meta-estimator (e.g. CrossValidator/TrainValidationSplit) case:
+          running CrossValidator.fit, the nested `estimator.fit` will be called in parallel,
+          but, the _autolog_training_status is a global status without thread-safe lock protecting.
+          This safe guarding will prevent code run into this case.
+        """
+        return not _SparkTrainingSession.is_active() and self._log_post_training_metrics_enabled
+
+    def disable_log_post_training_metrics(self):
+        class LogPostTrainingMetricsDisabledScope:
+            def __enter__(inner_self):  # pylint: disable=no-self-argument
+                # pylint: disable=attribute-defined-outside-init
+                inner_self.old_status = self._log_post_training_metrics_enabled
+                self._log_post_training_metrics_enabled = False
+
+            # pylint: disable=no-self-argument
+            def __exit__(inner_self, exc_type, exc_val, exc_tb):
+                self._log_post_training_metrics_enabled = inner_self.old_status
+
+        return LogPostTrainingMetricsDisabledScope()
+
+    @staticmethod
+    def get_run_id_for_model(model):
+        return getattr(model, "_mlflow_run_id", None)
+
+    @staticmethod
+    def is_metric_value_loggable(metric_value):
+        """
+        check whether the specified `metric_value` is a numeric value which can be logged
+        as an MLflow metric.
+        """
+        return isinstance(metric_value, (int, float, np.number)) and not isinstance(
+            metric_value, (bool, np.bool)
+        )
+
+    def register_model(self, model, run_id):
+        """
+        In `patched_fit`, we need register the model with the run_id used in `patched_fit`
+        So that in following metric autologging, the metric will be logged into the registered
+        run_id
+        """
+        model._mlflow_run_id = run_id
+
+    @staticmethod
+    def gen_name_with_index(name, index):
+        assert index >= 0
+        if index == 0:
+            return name
+        else:
+            # Use '-' as the separator between name and index,
+            # The '-' is not valid character in python var name
+            # so it can prevent name conflicts after appending index.
+            return f"{name}-{index + 1}"
+
+    def register_prediction_input_dataset(self, model, eval_dataset):
+        """
+        Register prediction input dataset into eval_dataset_info_map, it will do:
+         1. inspect eval dataset var name.
+         2. check whether eval_dataset_info_map already registered this eval dataset.
+            will check by object id.
+         3. register eval dataset with id.
+         4. return eval dataset name with index.
+
+        Note: this method include inspecting argument variable name.
+         So should be called directly from the "patched method", to ensure it capture
+         correct argument variable name.
+        """
+        eval_dataset_name = _inspect_original_var_name(
+            eval_dataset, fallback_name="unknown_dataset"
+        )
+        eval_dataset_id = id(eval_dataset)
+
+        run_id = self.get_run_id_for_model(model)
+        registered_dataset_list = self._eval_dataset_info_map[run_id][eval_dataset_name]
+
+        for i, id_i in enumerate(registered_dataset_list):
+            if eval_dataset_id == id_i:
+                index = i
+                break
+        else:
+            index = len(registered_dataset_list)
+
+        if index == len(registered_dataset_list):
+            # register new eval dataset
+            registered_dataset_list.append(eval_dataset_id)
+
+        return self.gen_name_with_index(eval_dataset_name, index)
+
+    def register_prediction_result(self, run_id, eval_dataset_name, predict_result):
+        """
+        Register the relationship
+         id(prediction_result) --> (eval_dataset_name, run_id)
+        into map `_pred_result_id_to_dataset_name_and_run_id`
+        """
+        value = (eval_dataset_name, run_id)
+        prediction_result_id = id(predict_result)
+        self._pred_result_id_to_dataset_name_and_run_id[prediction_result_id] = value
+
+        def clean_id(id_):
+            _AUTOLOGGING_METRICS_MANAGER._pred_result_id_to_dataset_name_and_run_id.pop(id_, None)
+
+        # When the `predict_result` object being GCed, its ID may be reused, so register a finalizer
+        # to clear the ID from the dict for preventing wrong ID mapping.
+        weakref.finalize(predict_result, clean_id, prediction_result_id)
+
+    def get_run_id_and_dataset_name_for_evaluator_call(self, pred_result_dataset):
+        """
+        Given a registered prediction result dataset object,
+        return a tuple of (run_id, eval_dataset_name)
+        """
+        if id(pred_result_dataset) in self._pred_result_id_to_dataset_name_and_run_id:
+            dataset_name, run_id = self._pred_result_id_to_dataset_name_and_run_id[
+                id(pred_result_dataset)
+            ]
+            return run_id, dataset_name
+        else:
+            return None, None
+
+    def gen_evaluator_info(self, evaluator):
+        """
+        Generate evaluator information, include evaluator class name and params.
+        """
+        class_name = _get_fully_qualified_class_name(evaluator)
+        param_map = _truncate_dict(
+            _get_param_map(evaluator), MAX_ENTITY_KEY_LENGTH, MAX_PARAM_VAL_LENGTH
+        )
+        return {"evaluator_class": class_name, "params": param_map}
+
+    def register_evaluator_call(self, run_id, metric_name, dataset_name, evaluator_info):
+        """
+        Register the `Evaluator.evaluate` call, including register the evaluator information
+        (See doc of `gen_evaluator_info` method) into the corresponding run_id and metric_name
+        entry in the registry table.
+        """
+        evaluator_call_info_list = self._evaluator_call_info[run_id][metric_name]
+
+        index = len(evaluator_call_info_list)
+        metric_name_with_index = self.gen_name_with_index(metric_name, index)
+        metric_key = f"{metric_name_with_index}_{dataset_name}"
+
+        evaluator_call_info_list.append((metric_key, evaluator_info))
+
+        # Set the flag to true, represent the metric info in this run need update.
+        # Later when `log_eval_metric` called, it will generate a new metric_info artifact
+        # and overwrite the old artifact.
+        self._metric_info_artifact_need_update[run_id] = True
+        return metric_key
+
+    def log_post_training_metric(self, run_id, key, value):
+        """
+        Log the metric into the specified mlflow run.
+        and it will also update the metric_info artifact if needed.
+        """
+        # Note: if the case log the same metric key multiple times,
+        #  newer value will overwrite old value
+        client = mlflow.tracking.MlflowClient()
+        client.log_metric(run_id=run_id, key=key, value=value)
+        if self._metric_info_artifact_need_update[run_id]:
+            evaluator_call_list = []
+            for v in self._evaluator_call_info[run_id].values():
+                evaluator_call_list.extend(v)
+
+            evaluator_call_list.sort(key=lambda x: x[0])
+            dict_to_log = OrderedDict(evaluator_call_list)
+            client.log_dict(run_id=run_id, dictionary=dict_to_log, artifact_file="metric_info.json")
+            self._metric_info_artifact_need_update[run_id] = False
+
+
+# The global `_AutologgingMetricsManager` instance which holds information used in
+# post-training metric autologging. See doc of class `_AutologgingMetricsManager` for details.
+_AUTOLOGGING_METRICS_MANAGER = _AutologgingMetricsManager()
+
+
 @experimental
 @autologging_integration(AUTOLOGGING_INTEGRATION_NAME)
 def autolog(
@@ -419,6 +681,7 @@ def autolog(
     exclusive=False,
     disable_for_unsupported_versions=False,
     silent=False,
+    log_post_training_metrics=True,
 ):  # pylint: disable=unused-argument
     """
     Enables (or disables) and configures autologging for pyspark ml estimators.
@@ -440,6 +703,33 @@ def autolog(
         - A fully qualified estimator class name
           (e.g. "pyspark.ml.regression.LinearRegression").
 
+      .. _post training metrics:
+
+      **Post training metrics**
+        When users call evaluator APIs after model training, MLflow tries to capture the
+        `Evaluator.evaluate` results and log them as MLflow metrics to the Run associated with
+        the model. All pyspark ML evaluators are supported.
+
+        For post training metrics autologging, the metric key format is:
+        "{metric_name}[-{call_index}]_{dataset_name}"
+
+        - The metric name is the name returned by `Evaluator.getMetricName()`
+        - If multiple calls are made to the same pyspark ML evaluator metric, each subsequent call
+          adds a "call_index" (starting from 2) to the metric key.
+        - MLflow uses the prediction input dataset variable name as the "dataset_name" in the
+          metric key. The "prediction input dataset variable" refers to the variable which was
+          used as the `dataset` argument of `model.transform` call.
+          Note: MLflow captures the "prediction input dataset" instance in the outermost call
+          frame and fetches the variable name in the outermost call frame. If the "prediction
+          input dataset" instance is an intermediate expression without a defined variable
+          name, the dataset name is set to "unknown_dataset". If multiple "prediction input
+          dataset" instances have the same variable name, then subsequent ones will append an
+          index (starting from 2) to the inspected dataset name.
+
+        **Limitations**
+          - MLflow cannot find run information for other objects derived from a given prediction
+            result (e.g. by doing some tranformation on the prediction result dataset).
+
       **Artifacts**
         - An MLflow Model with the :py:mod:`mlflow.spark` flavor containing a fitted estimator
           (logged by :py:func:`mlflow.spark.log_model()`). Note that large models may not be
@@ -447,6 +737,10 @@ def autolog(
           Pipelines and hyperparameter tuning meta-estimators (e.g. CrossValidator) is not yet
           supported.
           See ``log_models`` param below for details.
+        - For post training metrics API calls, a "metric_info.json" artifact is logged. This is a
+          JSON object whose keys are MLflow post training metric names
+          (see "Post training metrics" section for the key format) and whose values are the
+          corresponding evaluator information, including evaluator class name and evaluator params.
 
     **How does autologging work for meta estimators?**
           When a meta estimator (e.g. `Pipeline`_, `CrossValidator`_, `TrainValidationSplit`_,
@@ -499,13 +793,17 @@ def autolog(
     :param silent: If ``True``, suppress all event logs and warnings from MLflow during pyspark ML
                    autologging. If ``False``, show all events and warnings during pyspark ML
                    autologging.
+    :param log_post_training_metrics: If ``True``, post training metrics are logged. Defaults to
+                                      ``True``. See the `post training metrics`_ section for more
+                                      details.
 
     **The default log model allowlist in mlflow**
         .. literalinclude:: ../../../mlflow/pyspark/ml/log_model_allowlist.txt
            :language: text
     """
     from mlflow.tracking.context import registry as context_registry
-    from pyspark.ml.base import Estimator
+    from pyspark.ml.base import Estimator, Model
+    from pyspark.ml.evaluation import Evaluator
 
     global _log_model_allowlist
 
@@ -578,15 +876,12 @@ def autolog(
                 estimator, estimator._autologging_metadata.uid_to_indexed_name_map
             )
 
-            metric_key, metrics = _get_param_search_metrics(estimator, spark_model)
+            metrics_dict, best_index = _get_param_search_metrics_and_best_index(
+                estimator, spark_model
+            )
             _log_parameter_search_results_as_artifact(
-                estimator_param_maps, metrics, metric_key, mlflow.active_run().info.run_id
+                estimator_param_maps, metrics_dict, mlflow.active_run().info.run_id
             )
-
-            if estimator.getEvaluator().isLargerBetter():
-                best_index = np.argmax(metrics)
-            else:
-                best_index = np.argmin(metrics)
 
             # Log best_param_map as JSON artifact
             best_param_map = estimator_param_maps[best_index]
@@ -614,11 +909,7 @@ def autolog(
                 _logger.warning(_get_warning_msg_for_skip_log_model(spark_model))
 
     def fit_mlflow(original, self, *args, **kwargs):
-        params = None
-        if len(args) > 1:
-            params = args[1]
-        elif "params" in kwargs:
-            params = kwargs["params"]
+        params = get_method_call_arg_value(1, "params", None, args, kwargs)
 
         # Do not perform autologging on direct calls to fit() for featurizers.
         # Note that featurizers will be autologged when they're fit as part of a Pipeline.
@@ -630,19 +921,86 @@ def autolog(
             _logger.warning(_get_warning_msg_for_fit_call_with_a_list_of_params(self))
             return original(self, *args, **kwargs)
         else:
-            estimator = self.copy(params)
+            # we need generate estimator param map so we call `self.copy(params)` to construct
+            # an estimator with the extra params.
+            estimator = self.copy(params) if params is not None else self
             _log_pretraining_metadata(estimator, params)
             spark_model = original(self, *args, **kwargs)
             _log_posttraining_metadata(estimator, spark_model, params)
             return spark_model
 
     def patched_fit(original, self, *args, **kwargs):
+        should_log_post_training_metrics = (
+            log_post_training_metrics
+            and _AUTOLOGGING_METRICS_MANAGER.should_log_post_training_metrics()
+        )
         with _SparkTrainingSession(clazz=self.__class__, allow_children=False) as t:
             if t.should_log():
-                return fit_mlflow(original, self, *args, **kwargs)
+                with _AUTOLOGGING_METRICS_MANAGER.disable_log_post_training_metrics():
+                    fit_result = fit_mlflow(original, self, *args, **kwargs)
+                # In some cases the `fit_result` may be an iterator of spark models.
+                if should_log_post_training_metrics and isinstance(fit_result, Model):
+                    _AUTOLOGGING_METRICS_MANAGER.register_model(
+                        fit_result, mlflow.active_run().info.run_id
+                    )
+                return fit_result
             else:
                 return original(self, *args, **kwargs)
 
+    def patched_transform(original, self, *args, **kwargs):
+        run_id = _AUTOLOGGING_METRICS_MANAGER.get_run_id_for_model(self)
+        if _AUTOLOGGING_METRICS_MANAGER.should_log_post_training_metrics() and run_id:
+            predict_result = original(self, *args, **kwargs)
+            eval_dataset = get_method_call_arg_value(0, "dataset", None, args, kwargs)
+            eval_dataset_name = _AUTOLOGGING_METRICS_MANAGER.register_prediction_input_dataset(
+                self, eval_dataset
+            )
+            _AUTOLOGGING_METRICS_MANAGER.register_prediction_result(
+                run_id, eval_dataset_name, predict_result
+            )
+            return predict_result
+        else:
+            return original(self, *args, **kwargs)
+
+    def patched_evaluate(original, self, *args, **kwargs):
+        if _AUTOLOGGING_METRICS_MANAGER.should_log_post_training_metrics():
+            with _AUTOLOGGING_METRICS_MANAGER.disable_log_post_training_metrics():
+                metric = original(self, *args, **kwargs)
+
+            if _AUTOLOGGING_METRICS_MANAGER.is_metric_value_loggable(metric):
+                params = get_method_call_arg_value(1, "params", None, args, kwargs)
+                # we need generate evaluator param map so we call `self.copy(params)` to construct
+                # an evaluator with the extra evaluation params.
+                evaluator = self.copy(params) if params is not None else self
+                metric_name = evaluator.getMetricName()
+                evaluator_info = _AUTOLOGGING_METRICS_MANAGER.gen_evaluator_info(evaluator)
+
+                pred_result_dataset = get_method_call_arg_value(0, "dataset", None, args, kwargs)
+                (
+                    run_id,
+                    dataset_name,
+                ) = _AUTOLOGGING_METRICS_MANAGER.get_run_id_and_dataset_name_for_evaluator_call(
+                    pred_result_dataset
+                )
+                if run_id and dataset_name:
+                    metric_key = _AUTOLOGGING_METRICS_MANAGER.register_evaluator_call(
+                        run_id, metric_name, dataset_name, evaluator_info
+                    )
+                    _AUTOLOGGING_METRICS_MANAGER.log_post_training_metric(
+                        run_id, metric_key, metric
+                    )
+            return metric
+        else:
+            return original(self, *args, **kwargs)
+
     safe_patch(
         AUTOLOGGING_INTEGRATION_NAME, Estimator, "fit", patched_fit, manage_run=True,
     )
+
+    if log_post_training_metrics:
+        safe_patch(
+            AUTOLOGGING_INTEGRATION_NAME, Model, "transform", patched_transform, manage_run=False,
+        )
+        safe_patch(
+            AUTOLOGGING_INTEGRATION_NAME, Evaluator, "evaluate", patched_evaluate, manage_run=False,
+        )
