@@ -70,7 +70,14 @@ class __MLflowPLCallback(pl.Callback, metaclass=ExceptionSafeAbstractClass):
         # as a sanity check to catch bugs without having to wait for the training routine
         # to complete. During this check, we should skip logging metrics.
         # https://pytorch-lightning.readthedocs.io/en/stable/common/trainer.html#num-sanity-val-steps # noqa
-        if trainer.running_sanity_check:
+        sanity_checking = (
+            # `running_sanity_check` has been renamed to `sanity_checking`:
+            # https://github.com/PyTorchLightning/pytorch-lightning/pull/9209
+            trainer.sanity_checking
+            if Version(pl.__version__) > Version("1.4.5")
+            else trainer.running_sanity_check
+        )
+        if sanity_checking:
             return
 
         if (pl_module.current_epoch + 1) % self.log_every_n_epoch == 0:
@@ -288,7 +295,11 @@ def patched_fit(original, self, *args, **kwargs):
     if early_stop_callback is not None:
         _log_early_stop_metrics(early_stop_callback, client, run_id)
 
-    summary = str(ModelSummary(self.model, mode="full"))
+    if Version(pl.__version__) < Version("1.4.0"):
+        summary = str(ModelSummary(self.model, mode="full"))
+    else:
+        summary = str(ModelSummary(self.model, max_depth=-1))
+
     tempdir = tempfile.mkdtemp()
     try:
         summary_file = os.path.join(tempdir, "model_summary.txt")
