@@ -9,6 +9,7 @@ from abc import abstractmethod
 from contextlib import contextmanager
 
 import mlflow
+import mlflow.utils.autologging_utils
 from mlflow.entities.run_status import RunStatus
 from mlflow.tracking.client import MlflowClient
 from mlflow.utils import gorilla
@@ -22,6 +23,8 @@ from mlflow.utils.mlflow_tags import MLFLOW_AUTOLOGGING
 
 _AUTOLOGGING_TEST_MODE_ENV_VAR = "MLFLOW_AUTOLOGGING_TESTING"
 
+_AUTOLOGGING_PATCHES = {}
+
 
 def try_mlflow_log(fn, *args, **kwargs):
     """
@@ -179,7 +182,8 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
           with the `FAILED` if an unhandled exception is thrown during function execution
 
     Note that, if nested runs or non-fluent runs are created by `patch_function`, `patch_function`
-    is responsible for terminating them by the time it terminates (or in the event of an exception).
+    is responsible for terminating them by the time it terminates
+    (or in the event of an exception).
 
     :param autologging_integration: The autologging integration associated
                                     with the `patch_function`.
@@ -190,9 +194,7 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
     """
 
     def create_managed_run():
-        managed_run = mlflow.start_run()
-        if tags:
-            try_mlflow_log(mlflow.set_tags, tags)
+        managed_run = mlflow.start_run(tags=tags)
         _logger.info(
             "Created MLflow autologging run with ID '%s', which will track hyperparameters,"
             " performance metrics, model artifacts, and lineage information for the"
@@ -308,6 +310,41 @@ def safe_patch(
     else:
         assert callable(patch_function)
 
+    original_fn = gorilla.get_original_attribute(
+        destination, function_name, bypass_descriptor_protocol=False
+    )
+    # Retrieve raw attribute while bypassing the descriptor protocol
+    raw_original_obj = gorilla.get_original_attribute(
+        destination, function_name, bypass_descriptor_protocol=True
+    )
+    if original_fn != raw_original_obj:
+        raise RuntimeError(f"Unsupport patch on {str(destination)}.{function_name}")
+    elif isinstance(original_fn, property):
+        is_property_method = True
+
+        # For property decorated methods (a kind of method delegation), e.g.
+        # class A:
+        #   @property
+        #   def f1(self):
+        #     ...
+        #     return delegated_f1
+        #
+        # suppose `a1` is an instance of class `A`,
+        # `A.f1.fget` will get the original `def f1(self)` method,
+        # and `A.f1.fget(a1)` will be equivalent to `a1.f1()` and
+        # its return value will be the `delegated_f1` function.
+        # So using the `property.fget` we can construct the (delegated) "original_fn"
+        def original(self, *args, **kwargs):
+            # the `original_fn.fget` will get the original method decorated by `property`
+            # the `original_fn.fget(self)` will get the delegated function returned by the
+            # property decorated method.
+            bound_delegate_method = original_fn.fget(self)
+            return bound_delegate_method(*args, **kwargs)
+
+    else:
+        original = original_fn
+        is_property_method = False
+
     def safe_patch_function(*args, **kwargs):
         """
         A safe wrapper around the specified `patch_function` implementation designed to
@@ -334,10 +371,10 @@ def safe_patch(
         # during model serialization by ML frameworks such as scikit-learn
         is_silent_mode = get_autologging_config(autologging_integration, "silent", False)
         with set_mlflow_events_and_warnings_behavior_globally(
-            # MLflow warnings emitted during autologging training sessions are likely not actionable
-            # and result from the autologging implementation invoking another MLflow API.
-            # Accordingly, we reroute these warnings to the MLflow event logger with level WARNING
-            # For reference, see recommended warning and event logging behaviors from
+            # MLflow warnings emitted during autologging training sessions are likely not
+            # actionable and result from the autologging implementation invoking another MLflow
+            # API. Accordingly, we reroute these warnings to the MLflow event logger with level
+            # WARNING For reference, see recommended warning and event logging behaviors from
             # https://docs.python.org/3/howto/logging.html#when-to-use-logging
             reroute_warnings=True,
             disable_event_logs=is_silent_mode,
@@ -346,9 +383,9 @@ def safe_patch(
             # non-MLflow Warnings emitted during the autologging preamble (before the original /
             # underlying ML function is called) and postamble (after the original / underlying ML
             # function is called) are likely not actionable and result from the autologging
-            # implementation invoking an API from a dependent library. Accordingly, we reroute these
-            # warnings to the MLflow event logger with level WARNING. For reference, see recommended
-            # warning and event logging behaviors from
+            # implementation invoking an API from a dependent library. Accordingly, we reroute
+            # these warnings to the MLflow event logger with level WARNING. For reference, see
+            # recommended warning and event logging behaviors from
             # https://docs.python.org/3/howto/logging.html#when-to-use-logging
             reroute_warnings=True,
             disable_warnings=is_silent_mode,
@@ -357,8 +394,6 @@ def safe_patch(
             if is_testing():
                 preexisting_run_for_testing = mlflow.active_run()
 
-            original = gorilla.get_original_attribute(destination, function_name)
-
             # Whether or not to exclude autologged content from user-created fluent runs
             # (i.e. runs created manually via `mlflow.start_run()`)
             exclusive = get_autologging_config(autologging_integration, "exclusive", False)
@@ -374,6 +409,7 @@ def safe_patch(
                 active_session_failed
                 or autologging_is_disabled(autologging_integration)
                 or (user_created_fluent_run_is_active and exclusive)
+                or mlflow.utils.autologging_utils._AUTOLOGGING_GLOBALLY_DISABLED
             ):
                 # If the autologging integration associated with this patch is disabled,
                 # or if the current autologging integration is in exclusive mode and a user-created
@@ -396,6 +432,8 @@ def safe_patch(
             failed_during_original = False
             # The active MLflow run (if any) associated with patch code execution
             patch_function_run_for_testing = None
+            # The exception raised during executing patching function
+            patch_function_exception = None
 
             def try_log_autologging_event(log_fn, *args):
                 try:
@@ -405,20 +443,47 @@ def safe_patch(
                         "Failed to log autologging event via '%s'. Exception: %s", log_fn, e,
                     )
 
+            def call_original_fn_with_event_logging(original_fn, og_args, og_kwargs):
+                try:
+                    try_log_autologging_event(
+                        AutologgingEventLogger.get_logger().log_original_function_start,
+                        session,
+                        destination,
+                        function_name,
+                        og_args,
+                        og_kwargs,
+                    )
+                    original_fn_result = original_fn(*og_args, **og_kwargs)
+
+                    try_log_autologging_event(
+                        AutologgingEventLogger.get_logger().log_original_function_success,
+                        session,
+                        destination,
+                        function_name,
+                        og_args,
+                        og_kwargs,
+                    )
+                    return original_fn_result
+                except Exception as original_fn_e:
+                    try_log_autologging_event(
+                        AutologgingEventLogger.get_logger().log_original_function_error,
+                        session,
+                        destination,
+                        function_name,
+                        og_args,
+                        og_kwargs,
+                        original_fn_e,
+                    )
+
+                    nonlocal failed_during_original
+                    failed_during_original = True
+                    raise
+
             with _AutologgingSessionManager.start_session(autologging_integration) as session:
                 try:
 
                     def call_original(*og_args, **og_kwargs):
-                        try:
-                            try_log_autologging_event(
-                                AutologgingEventLogger.get_logger().log_original_function_start,
-                                session,
-                                destination,
-                                function_name,
-                                og_args,
-                                og_kwargs,
-                            )
-
+                        def _original_fn(*_og_args, **_og_kwargs):
                             if is_testing():
                                 _validate_args(args, kwargs, og_args, og_kwargs)
                                 # By the time `original` is called by the patch implementation, we
@@ -435,39 +500,17 @@ def safe_patch(
                             original_has_been_called = True
 
                             nonlocal original_result
-                            # Show all non-MLflow warnings as normal (i.e. not as event logs) during
-                            # original function execution, even if silent mode is enabled
+                            # Show all non-MLflow warnings as normal (i.e. not as event logs)
+                            # during original function execution, even if silent mode is enabled
                             # (`silent=True`), since these warnings originate from the ML framework
                             # or one of its dependencies and are likely relevant to the caller
                             with set_non_mlflow_warnings_behavior_for_current_thread(
                                 disable_warnings=False, reroute_warnings=False,
                             ):
-                                original_result = original(*og_args, **og_kwargs)
+                                original_result = original(*_og_args, **_og_kwargs)
+                                return original_result
 
-                            try_log_autologging_event(
-                                AutologgingEventLogger.get_logger().log_original_function_success,
-                                session,
-                                destination,
-                                function_name,
-                                og_args,
-                                og_kwargs,
-                            )
-
-                            return original_result
-                        except Exception as e:
-                            try_log_autologging_event(
-                                AutologgingEventLogger.get_logger().log_original_function_error,
-                                session,
-                                destination,
-                                function_name,
-                                og_args,
-                                og_kwargs,
-                                e,
-                            )
-
-                            nonlocal failed_during_original
-                            failed_during_original = True
-                            raise
+                        return call_original_fn_with_event_logging(_original_fn, og_args, og_kwargs)
 
                     # Apply the name, docstring, and signature of `original` to `call_original`.
                     # This is important because several autologging patch implementations inspect
@@ -500,29 +543,13 @@ def safe_patch(
                     )
                 except Exception as e:
                     session.state = "failed"
-
+                    patch_function_exception = e
                     # Exceptions thrown during execution of the original function should be
                     # propagated to the caller. Additionally, exceptions encountered during test
                     # mode should be reraised to detect bugs in autologging implementations
                     if failed_during_original or is_testing():
                         raise
 
-                    try_log_autologging_event(
-                        AutologgingEventLogger.get_logger().log_patch_function_error,
-                        session,
-                        destination,
-                        function_name,
-                        args,
-                        kwargs,
-                        e,
-                    )
-
-                    _logger.warning(
-                        "Encountered unexpected error during %s autologging: %s",
-                        autologging_integration,
-                        e,
-                    )
-
                 if is_testing() and not preexisting_run_for_testing:
                     # If an MLflow run was created during the execution of patch code, verify that
                     # it is no longer active and that it contains expected autologging tags
@@ -533,13 +560,89 @@ def safe_patch(
                         _validate_autologging_run(
                             autologging_integration, patch_function_run_for_testing.info.run_id
                         )
+                try:
+                    if original_has_been_called:
+                        return original_result
+                    else:
+                        return call_original_fn_with_event_logging(original, args, kwargs)
+                finally:
+                    # If original function succeeds, but `patch_function_exception` exists,
+                    # it represent patching code unexpected failure, so we call
+                    # `log_patch_function_error` in this case.
+                    # If original function failed, we don't call `log_patch_function_error`
+                    # even if `patch_function_exception` exists, because original function failure
+                    # means there's some error in user code (e.g. user provide wrong arguments)
+                    if patch_function_exception is not None and not failed_during_original:
+                        try_log_autologging_event(
+                            AutologgingEventLogger.get_logger().log_patch_function_error,
+                            session,
+                            destination,
+                            function_name,
+                            args,
+                            kwargs,
+                            patch_function_exception,
+                        )
 
-                if original_has_been_called:
-                    return original_result
-                else:
-                    return original(*args, **kwargs)
+                        _logger.warning(
+                            "Encountered unexpected error during %s autologging: %s",
+                            autologging_integration,
+                            patch_function_exception,
+                        )
 
-    _wrap_patch(destination, function_name, safe_patch_function)
+    if is_property_method:
+        # Create a patched function (also property decorated)
+        # like:
+        #
+        # class A:
+        # @property
+        # def get_bound_safe_patch_fn(self):
+        #   original_fn.fget(self) # do availability check
+        #   return bound_safe_patch_fn
+        #
+        # Suppose `a1` is instance of class A,
+        # then `a1.get_bound_safe_patch_fn(*args, **kwargs)` will be equivalent to
+        # `bound_safe_patch_fn(*args, **kwargs)`
+        def get_bound_safe_patch_fn(self):
+            # This `original_fn.fget` call is for availability check, if it raise error
+            # then `hasattr(obj, {func_name})` will return False
+            # so it mimic the original property behavior.
+            original_fn.fget(self)
+
+            def bound_safe_patch_fn(*args, **kwargs):
+                return safe_patch_function(self, *args, **kwargs)
+
+            # Make bound method `instance.target_method` keep the same doc and signature
+            bound_safe_patch_fn = update_wrapper_extended(bound_safe_patch_fn, original_fn.fget)
+            # Here return the bound safe patch function because user call property decorated
+            # method will like `instance.property_decorated_method(...)`, and internally it will
+            # call the `bound_safe_patch_fn`, the argument list don't include the `self` argument,
+            # so return bound function here.
+            return bound_safe_patch_fn
+
+        # Make unbound method `class.target_method` keep the same doc and signature
+        get_bound_safe_patch_fn = update_wrapper_extended(get_bound_safe_patch_fn, original_fn.fget)
+        safe_patch_obj = property(get_bound_safe_patch_fn)
+    else:
+        safe_patch_obj = update_wrapper_extended(safe_patch_function, original)
+
+    new_patch = _wrap_patch(destination, function_name, safe_patch_obj)
+    _store_patch(autologging_integration, new_patch)
+
+
+def revert_patches(autologging_integration):
+    """
+    Reverts all patches on the specified destination class for autologging disablement
+    purposes.
+
+    :param autologging_integration: The name of the autologging integration associated with the
+                                    patch. Note: If called via fluent api
+                                    (`autologging_integration="mlfow"`), then revert all patches
+                                    for all active autologging integrations.
+    """
+    for patch in _AUTOLOGGING_PATCHES.get(autologging_integration, []):
+        gorilla.revert(patch)
+
+    _AUTOLOGGING_PATCHES.pop(autologging_integration, None)
 
 
 # Represents an active autologging session using two fields:
@@ -601,23 +704,37 @@ def update_wrapper_extended(wrapper, wrapped):
     return updated_wrapper
 
 
-def _wrap_patch(destination, name, patch, settings=None):
+def _wrap_patch(destination, name, patch_obj, settings=None):
     """
-    Apply a patch while preserving the attributes (e.g. __doc__) of an original function.
+    Apply a patch.
 
     :param destination: Patch destination
     :param name: Name of the attribute at the destination
-    :param patch: Patch function
+    :param patch_obj: Patch object, it should be a function or a property decorated function
+                      to be assigned to the patch point {destination}.{name}
     :param settings: Settings for gorilla.Patch
     """
     if settings is None:
         settings = gorilla.Settings(allow_hit=True, store_hit=True)
 
-    original = getattr(destination, name)
-    wrapped = update_wrapper_extended(patch, original)
-
-    patch = gorilla.Patch(destination, name, wrapped, settings=settings)
+    patch = gorilla.Patch(destination, name, patch_obj, settings=settings)
     gorilla.apply(patch)
+    return patch
+
+
+def _store_patch(autologging_integration, patch):
+    """
+    Stores a patch for a specified autologging_integration class. Later to be used for being able
+    to revert the patch when disabling autologging.
+
+    :param autologging_integration: The name of the autologging integration associated with the
+                                    patch.
+    :param patch: The patch to be stored.
+    """
+    if autologging_integration in _AUTOLOGGING_PATCHES:
+        _AUTOLOGGING_PATCHES[autologging_integration].add(patch)
+    else:
+        _AUTOLOGGING_PATCHES[autologging_integration] = set([patch])
 
 
 def _validate_autologging_run(autologging_integration, run_id):
